<?php

namespace App\Factory;

use App\Exception;
use Doctrine\DBAL\Types\StringType;

use function DI\string;

class ProductFactory
{
    private $connection;
    private $util;

    public function __construct(
        \Doctrine\DBAL\Connection $connection,
        \App\Util\Util $util
    ) {
        $this->connection = $connection;
        $this->util = $util;
    }

    public function getProduct($product)
    {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDPRODUTO  = :CDPRODUTO
                OR CDPRODINTE = :CDPRODINTE
                OR CDARVPROD  = :CDARVPROD
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);


        $params = [
            ':CDPRODUTO'  => $productCode,
            ':CDPRODINTE' => $productCode,
            ":CDARVPROD"  => $productCode
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getObservation($observation, $isAcr = false)
    {
        if (strlen($observation) != 5 && substr($observation, 2, 1) != '|')
            throw new \Exception("Erro ao buscar observação. {$observation} não é um código de observação válido");

        if ($isAcr) {
            $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS = 'A'
        		  AND CDPRODUTO IS NOT NULL
            ";
        } else {
            $sql = "
        		SELECT *
        		FROM OCORRENCIA
        		WHERE CDGRPOCOR = :CDGRPOCOR
        		  AND CDOCORR   = :CDOCORR
        		  AND IDCONTROLAOBS IN ('O', 'R')
            ";
        }

        $obs = explode("|", $observation);

        $params = [
            ":CDGRPOCOR" => $obs[0],
            ":CDOCORR"   => $obs[1]
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function insertProduct($product, $generalParams)
    {
        $sql = "
        INSERT INTO ITCOMANDAVEN (
            CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, 
            TXPRODCOMVEN, IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, 
            NRSEQPRODCOM, NRSEQPRODCUP, VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, 
            NRINSCRCONS, CDSENHAPED, NRATRAPRODCOVE, IDORIGEMVENDA, DSOBSPEDDIGCMD, IDIMPPRGRPROMO,
            DTHRINCOMVEN, IDOBSERVACAOAD, IDPRODEXT
          )
          VALUES (
            :CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, 
            :TXPRODCOMVEN, :IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, :IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, 
            :NRSEQPRODCOM, :NRSEQPRODCUP, :VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, 
            :NRINSCRCONS, :CDSENHAPED, :NRATRAPRODCOVE, :IDORIGEMVENDA, :DSOBSPEDDIGCMD, :IDIMPPRGRPROMO,
            GETDATE(), :IDOBSERVACAOAD, :IDPRODEXT)
        ";

        if (isset($generalParams["SGESTADO"]))
            date_default_timezone_set($this->util->getCurrentTimezone($generalParams["SGESTADO"]));

        $params = [
            ':CDFILIAL'       => $product['CDFILIAL'],
            ':NRVENDAREST'    => $product['NRVENDAREST'],
            ':NRCOMANDA'      => $product['NRCOMANDA'],
            ':NRPRODCOMVEN'   => $product['NRPRODCOMVEN'],
            ':CDPRODUTO'      => $product['CDPRODUTO'],
            ':QTPRODCOMVEN'   => $this->util->toFloat($product['QTPRODCOMVEN']),
            ':VRPRECCOMVEN'   => $this->util->toFloat($product['VRPRECCOMVEN']),
            ':TXPRODCOMVEN'   => $product['TXPRODCOMVEN'],
            ':IDSTPRCOMVEN'   => $product['IDSTPRCOMVEN'],
            ':VRDESCCOMVEN'   => $product['VRDESCCOMVEN'],
            ':NRLUGARMESA'    => $product['NRLUGARMESA'],
            ':IDPRODIMPFIS'   => $product['IDPRODIMPFIS'],
            ':CDLOJA'         => $product['CDLOJA'],
            ':VRACRCOMVEN'    => $product['VRACRCOMVEN'],
            ':NRSEQPRODCOM'   => $product['NRSEQPRODCOM'],
            ':NRSEQPRODCUP'   => $product['NRSEQPRODCUP'],
            ':VRPRECCLCOMVEN' => $product['VRPRECCLCOMVEN'],
            ':CDCAIXACOLETOR' => $product['CDCAIXACOLETOR'],
            ':CDPRODPROMOCAO' => $product['CDPRODPROMOCAO'],
            ':CDVENDEDOR'     => $product['CDVENDEDOR'],
            ':NRINSCRCONS'    => $product['NRINSCRCONS'],
            ':CDSENHAPED'     => $this->util->toRight($product['CDSENHAPED'], 5),
            ':NRATRAPRODCOVE' => 0,
            ':IDORIGEMVENDA'  => null,
            ':DSOBSPEDDIGCMD' => $product['DSOBSPEDDIGCMD'],
            ':IDIMPPRGRPROMO' => 'N',
            ':IDOBSERVACAOAD' => (!empty($product['IDOBSERVACAOAD']) ? $product['IDOBSERVACAOAD'] : 'N'),
            ':IDPRODEXT'      => $product['IDPRODEXT']
        ];

        $this->connection->executeQuery($sql, $params);


        return $product['NRPRODCOMVEN'];
    }

    public function insertItComandaEst($product, $generalParams)
    {
        $sql = "
        INSERT INTO ITCOMANDAEST (
            CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDPRODUTO, QTPROCOMEST, VRPRECCOMEST, 
            VRDESITCOMEST, IDIMPPRGRPROMO, VRACRITCOMEST, TXPRODCOMVENEST, IDPRODEXT
          )
          VALUES (
            :CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDPRODUTO, :QTPROCOMEST, :VRPRECCOMEST, 
            :VRDESITCOMEST, :IDIMPPRGRPROMO, :VRACRITCOMEST, :TXPRODCOMVENEST, :IDPRODEXT
          )
        ";

        
        $params = [
            ':CDFILIAL'        => $product['CDFILIAL'],
            ':NRVENDAREST'     => $product['NRVENDAREST'],
            ':NRCOMANDA'       => $product['NRCOMANDA'],
            ':NRPRODCOMVEN'    => $product['NRPRODCOMVEN'],
            ':CDPRODUTO'       => $product['CDPRODUTO'],
            ':QTPROCOMEST'     => $product['QTPROCOMEST'],
            ':VRPRECCOMEST'    => $product['VRPRECCOMEST'],
            ':VRDESITCOMEST'   => $product['VRDESITCOMEST'],
            ':IDIMPPRGRPROMO'  => $product['IDIMPPRGRPROMO'],
            ':VRACRITCOMEST'   => $product['VRACRITCOMEST'],
            ':TXPRODCOMVENEST' => $product['TXPRODCOMVENEST'],
            ':IDPRODEXT'       => $product['IDPRODEXT'],
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function updateItComandaEst($product, $generalParams)
    {
        $sql = "
            UPDATE ITCOMANDAEST
                SET 
                    QTPROCOMEST  = :QTPROCOMEST,
                    IDPRODEXT    = :IDPRODEXT
            WHERE CDFILIAL       = :CDFILIAL
                AND CDPRODUTO    = :CDPRODUTO
                AND NRVENDAREST  = :NRVENDAREST
                AND NRCOMANDA    = :NRCOMANDA
                AND NRPRODCOMVEN = :NRPRODCOMVEN
        ";
        $params = [
            ':QTPROCOMEST'     => $product['QTPROCOMEST'],
            ':TXPRODCOMVENEST' => $product['TXPRODCOMVENEST'],
            ':CDFILIAL'        => $product['CDFILIAL'],
            ':CDPRODUTO'       => $product['CDPRODUTO'],
            ':NRVENDAREST'     => $product['NRVENDAREST'],
            ':NRCOMANDA'       => $product['NRCOMANDA'],
            ':NRPRODCOMVEN'    => $product['NRPRODCOMVEN'],
            ':IDPRODEXT'       => $product['IDPRODEXT']
        ];

        $this->connection->executeQuery($sql, $params);
    }

    public function insertObservation($observation, $generalParams, $orderData)
    {

        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)
        ";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $orderData["NRVENDAREST"],
            ":NRCOMANDA"       => $orderData["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $observation["NRPRODCOMVEN"],
            ":NRPRODCOMVENOBS" => $observation["NRPRODCOMVENOBS"],
            ":CDOCORR"         => $observation["CDOCORR"],
            ":CDGRPOCOR"       => $observation["CDGRPOCOR"],
            ":QTPRODCOMVENOBS" => $observation["QTPRODCOMVENOBS"]
        ];

        $this->connection->executeQuery($sql, $params);
    }

    public function insertObsItComandaEst($observation, $generalParams, $orderData)
    {

        $sql = "
        INSERT INTO OBSITCOMANDAEST (
            CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDPRODUTO, CDGRPOCOR, CDOCORR
          )
          VALUES (
            :CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDPRODUTO, :CDGRPOCOR, :CDOCORR
          )
        ";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $orderData["NRVENDAREST"],
            ":NRCOMANDA"       => $orderData["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $observation["NRPRODCOMVEN"],
            ":CDPRODUTO"       => $observation["CDPRODUTO"],
            ":CDOCORR"         => $observation["CDOCORR"],
            ":CDGRPOCOR"       => $observation["CDGRPOCOR"],
        ];

        $this->connection->executeQuery($sql, $params);
    }


    public function getNrProdComVen($nrVendarest, $nrComanda)
    {

        $sql = "
            SELECT MAX(NRPRODCOMVEN) AS NRPRODCOMVEN
	            FROM ITCOMANDAVEN
                    WHERE NRCOMANDA = :NRCOMANDA
                     AND NRVENDAREST = :NRVENDAREST
        ";
        $params = [
            ':NRCOMANDA'   => $nrComanda,
            ':NRVENDAREST' => $nrVendarest
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getTabePrec($cdFilial)
    {
        $sql = "
            SELECT T.CDTABEPREC AS CDTABEPRECGER
                FROM TABEVENDGER T, FILITABGER FT
                    WHERE (FT.CDFILIAL = :CDFILIAL)
                     AND (T.CDTABEPREC = FT.CDTABEPREC)
                     AND (CONVERT(DATE, GETUTCDATE(), 120) BETWEEN DTINIVGPREC AND DTFINVGPREC)
        ";
        $params = [
            ':CDFILIAL' => $cdFilial
        ];

        $CDTABEPRECGER = $this->connection->fetchAll($sql, $params);
        $sql = "
            SELECT DISTINCT T.CDTABEPREC AS CDTABEPRECFILI
                FROM TABEVEND T, PARAVEND PV, LOJA L
                    WHERE (T.CDTABEPREC = L.CDTABEPREC OR T.CDTABEPREC = PV.CDTABEPREC)
                     AND (CONVERT(DATE, GETUTCDATE(), 120) BETWEEN DTINIVGPREC AND DTFINVGPREC)
        ";

        $CDTABEPRECFILI = $this->connection->fetchAll($sql);

        if (count($CDTABEPRECGER) > 1 || count($CDTABEPRECFILI) > 1)
            throw new \Exception("Erro ao buscar tabela de preços. Encontrado mais de uma tabela parametrizado para Unidade.");

        $retorno = [
            'CDTABEPRECFILI' => isset($CDTABEPRECFILI[0]['CDTABEPRECFILI']) ? $CDTABEPRECFILI[0]['CDTABEPRECFILI'] : null,
            'CDTABEPRECGER'  => isset($CDTABEPRECGER[0]['CDTABEPRECGER']) ? $CDTABEPRECGER[0]['CDTABEPRECGER'] : null
        ];

        return $retorno;
    }

    public function ProductsForShipping($cdFilial, $cdTabePrec)
    {
        $sql = "
            SELECT P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, I.VRPRECITEM AS PRECOFILI, ITG.VRPRECITEM AS PRECOGER
                FROM PRODUTO P
                    LEFT JOIN ITEMPRECO I 
                        ON P.CDPRODUTO = I.CDPRODUTO
                    LEFT JOIN ITEMPRECOGER ITG
                        ON P.CDPRODUTO = ITG.CDPRODUTO
                    WHERE P.CDPRODUTO IN ( SELECT DISTINCT PRECO.CDPRODUTO
                                            FROM  (SELECT ITG.CDPRODUTO
                                                    FROM ITEMPRECOGER ITG
                                                        WHERE (ITG.CDTABEPREC = :CDTABEPRECGER)
                                                        AND (ITG.VRPRECITEM > 0)
                                                    UNION
                                                        SELECT IT.CDPRODUTO
                                                                FROM ITEMPRECO IT
                                                                WHERE (IT.CDFILIAL = :CDFILIAL) 
                                                                AND (IT.CDTABEPREC = :CDTABEPRECFILI)        
                                                                AND (IT.VRPRECITEM > 0)) PRECO)
                    AND ((I.CDTABEPREC = :CDTABEPRECFILI) OR (ITG.CDTABEPREC = :CDTABEPRECGER))
        ";

        $params = [
            ':CDFILIAL'       => $cdFilial,
            ':CDTABEPRECFILI' => $cdTabePrec['CDTABEPRECFILI'],
            ':CDTABEPRECGER'  => $cdTabePrec['CDTABEPRECGER']
        ];

        return $this->connection->fetchAll($sql, $params);
    }

    public function observationsForShipping($cdFilial)
    {

        $arrayObs = [];
        $sql = "
            SELECT CDGRPOCOR, CDOCORR, DSOCORR, 
                   CDPRODUTO, IDCONTROLAOBS  
                FROM OCORRENCIA
     
        ";

        $observations = $this->connection->fetchAll($sql);

        if (!empty($observations)) {
            foreach ($observations as $obs) {
                if (!empty($obs['CDPRODUTO'])) {
                    $data = $this->checkPriceItem($obs['CDPRODUTO'], $cdFilial);
                }

                $obsFormat = [
                    'CDARVPROD' => "{$obs['CDGRPOCOR']}|{$obs['CDOCORR']}",
                    'NMPRODUTO' => $obs['DSOCORR'],
                    'PRECOFILI' => isset($data['PRECOFILI']) ? $data['PRECOFILI'] : 0,
                    'PRECOGER'  => isset($data['PRECOGER']) ? $data['PRECOGER'] : 0
                ];

                array_push($arrayObs, $obsFormat);
                unset($data);
            }
        }

        return $arrayObs;
    }

    public function rodizioProductsForShipping($cdFilial)
    {
        $sql = "
            SELECT DISTINCT PGP.CDPRODUTO, P.NMPRODUTO, P.CDARVPROD FROM PRODGRUPROMOC PGP
                JOIN PRODUTO P
                    ON P.CDPRODUTO = PGP.CDPRODUTO
                LEFT JOIN GRUPROMOCPROD GPP
                    ON GPP.CDGRUPROMOC = PGP.CDGRUPROMOC
                    AND GPP.CDPRODPROMOCAO IN (SELECT CDPRODUTO FROM PRODUTO
                                                    WHERE IDTIPOCOMPPROD = '6')
                LEFT JOIN GRUPROMOCPRFIL GPF
                    ON GPF.CDGRUPROMOC = PGP.CDGRUPROMOC
                    AND GPF.CDFILIAL = :CDFILIAL
                    AND GPF.CDPRODPROMOCAO IN (SELECT CDPRODUTO FROM PRODUTO
                                                    WHERE IDTIPOCOMPPROD = '6')
                WHERE (PGP.CDGRUPROMOC = GPP.CDGRUPROMOC OR PGP.CDGRUPROMOC = GPF.CDGRUPROMOC)
        ";

        $params = [
            ':CDFILIAL'       => $cdFilial
        ];

        return $this->connection->fetchAll($sql, $params);
    }

    private function checkPriceItem($cdproduto, $cdFilial)
    {
        $sql = "
            SELECT P.CDPRODUTO, P.NMPRODUTO, I.VRPRECITEM AS PRECOFILI, ITG.VRPRECITEM AS PRECOGER
            FROM PRODUTO P
                LEFT JOIN ITEMPRECO I 
                    ON P.CDPRODUTO = I.CDPRODUTO
                    AND (I.CDTABEPREC IN (SELECT T.CDTABEPREC
                                            FROM TABEVEND T, PARAVEND PV, LOJA L
                                                WHERE (T.CDTABEPREC = L.CDTABEPREC OR T.CDTABEPREC = PV.CDTABEPREC)
                                                    AND (CONVERT(DATE, GETUTCDATE(), 120) BETWEEN DTINIVGPREC AND DTFINVGPREC)
                        ))
                LEFT JOIN ITEMPRECOGER ITG
                    ON P.CDPRODUTO = ITG.CDPRODUTO
                    AND (ITG.CDTABEPREC IN (SELECT T.CDTABEPREC
                                                FROM TABEVENDGER T, FILITABGER FT
                                                WHERE (FT.CDFILIAL = :CDFILIAL)
                                                    AND (T.CDTABEPREC = FT.CDTABEPREC)
                                                    AND (CONVERT(DATE, GETUTCDATE(), 120) BETWEEN DTINIVGPREC AND DTFINVGPREC)
                                                ))
                WHERE P.CDPRODUTO = :CDPRODUTO
        ";

        $params = [
            ':CDPRODUTO' => $cdproduto,
            ':CDFILIAL'  => $cdFilial
        ];

        $result = $this->connection->fetchAll($sql, $params);

        $result[0]['PRECOFILI'] = !empty($result[0]['PRECOFILI']) ? $result[0]['PRECOFILI'] : 0;
        $result[0]['PRECOGER']  = !empty($result[0]['PRECOGER']) ? $result[0]['PRECOGER'] : 0;

        return $result[0];
    }

    public function getMainProduct($prodExt, $cdFilial, $nrVendarest, $nrComanda)
    {
        $sql = "
            SELECT NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, NRSEQPRODCOM
                FROM ITCOMANDAVEN
            WHERE CDFILIAL = :CDFILIAL
            AND NRVENDAREST = :NRVENDAREST
            AND NRCOMANDA = :NRCOMANDA
            AND IDPRODEXT = :IDPRODEXT
        ";

        $params = [
            ':CDFILIAL'    => $cdFilial,
            ':NRVENDAREST' => $nrVendarest,
            ':NRCOMANDA'   => $nrComanda,
            ':IDPRODEXT'   => $prodExt
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getItcomandaEstProduct($item)
    {
        $sql = "
            SELECT TOP 1 * FROM ITCOMANDAEST
                WHERE CDFILIAL = :CDFILIAL
                AND NRVENDAREST = :NRVENDAREST
                AND NRCOMANDA = :NRCOMANDA
                AND NRPRODCOMVEN = :NRPRODCOMVEN
                AND CDPRODUTO = :CDPRODUTO
        ";

        $params = [
            ':CDFILIAL'     => $item['CDFILIAL'],
            ':NRVENDAREST'  => $item['NRVENDAREST'],
            ':NRCOMANDA'    => $item['NRCOMANDA'],
            ':NRPRODCOMVEN' => $item['NRPRODCOMVEN'],
            ':CDPRODUTO'    => $item['CDPRODUTO']
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
}
