<?php

namespace App\Service;

use App\Exception\Exception;

class IntegrationService
{

    private $integrationFactory;
    private $orderService;
    private $productService;
    private $printService;
    private $util;
    private $connection;

    public function __construct(
        \App\Factory\IntegrationFactory $integrationFactory,
        \App\Service\OrderService $orderService,
        \App\Service\ProductService $productService,
        \App\Service\PrintService $printService,
        \Doctrine\DBAL\Connection $connection,
        \App\Util\Util $util
    ) {
        $this->integrationFactory = $integrationFactory;
        $this->orderService = $orderService;
        $this->productService = $productService;
        $this->printService = $printService;
        $this->connection = $connection;
        $this->util = $util;
        $this->logService = $this->logPath() . '/Services';
    }

    private function logPath()
    {
        global $CLIENT_NAME;
        return $CLIENT_NAME['logPath'];
    }

    public function getGeneralParams()
    {
        $filial = $this->integrationFactory->getFilialDetails();
        $seller = $this->integrationFactory->getStandardSeller($filial['CDFILIAL']);

        if (!isset($seller) || empty($seller)) {
            throw new \Exception("Vendedor padrão não cadastrado para a Unidade {$filial['CDFILIAL']} - {$filial['NMFILIAL']}.");
        } elseif (empty($seller['CDOPERADOR'])) {
            throw new \Exception("Nenhum operador vinculado ao vendedor padrão {$seller['CDVENDEDOR']} - {$seller['NMFANVEN']}.");
        } elseif (empty($seller['NRMESA'])) {
            throw new \Exception("Mesa padrão não parametrizada para a loja.");
        } elseif (empty($seller['CDCAIXA'])) {
            throw new \Exception("Nenhum caixa vinculado ao vendedor padrão {$seller['CDVENDEDOR']} - {$seller['NMFANVEN']}.");
        }

        return $seller;
    }

    public function ProcessOrder($order)
    {

        // Validação de modalidade e abertura de consumação

        $this->connection->beginTransaction();

        try {
            $generalParams = $this->getGeneralParams();
            $cdFilial = $generalParams['CDFILIAL'];
            $cdLoja = $generalParams['CDLOJA'];
            $nrMesa = isset($order['table']) ? $this->util->toRight($order['table'], 4) : null;
            $nrComanda = isset($order['card']) ? $order['card'] : null;
            $fileName = date("Ymd-H-i-s") . "-" . $nrMesa . $nrComanda;

            $idControProd = $this->orderService->getIdcontroprod($cdLoja, $cdFilial);
            $IDSTCOMANDA  = '1';
            if ($idControProd === 'S')
                $IDSTCOMANDA = '1';

            if ($nrComanda > 0)
                $nrComanda = $this->util->toRight($nrComanda, 10);

            if (empty($nrMesa) && empty($nrComanda)) {
                throw new \Exception("Pedido sem dados da mesa e comanda. É obrigatório o envio de pelo menos um dos parâmetros.");
            }

            if (!empty($nrMesa)) {
                if (empty($nrComanda)) {
                    $mesa = $this->orderService->getTableIsOpen($nrMesa);
                    $orderType = 'MES_MOB';
                    $mode = 'MESA';
                }
            }

            if (!empty($nrComanda) && $nrComanda > 0) {
                $comanda = $this->orderService->getComanda($nrComanda);
                $orderType = 'CMD_MOB';
                $mode = 'COMANDA';
            }

            $type = substr($orderType, 0, 3);
            $qtdPeople = isset($order['people']) ? $order['people'] : 1;

            if ($type == 'CMD' && empty($comanda)) {
                $dsComanda = $nrComanda;
                $modImpressao = 'C';
                $orderData = $this->orderService->openConsummation($cdFilial, $cdLoja, $generalParams, $dsComanda, $nrMesa, $orderType, $IDSTCOMANDA, $qtdPeople);
            } elseif ($type == 'MES' && empty($mesa)) {
                if ($nrMesa == $generalParams['NRMESA'])
                    throw new \Exception("Não é possível utilizar a mesa padrão na modalidade Mesa.");

                $tableStatus = $this->orderService->checkTableStatus($cdFilial, $nrMesa, $cdLoja);
                if (isset($tableStatus['statusId']) && $tableStatus['statusId'] == 'A')
                    throw new \Exception("Falha na abertura da mesa {$nrMesa}. Mesa Indisponível.");

                $orderData = $this->orderService->openConsummation($cdFilial, $cdLoja, $generalParams, null, $nrMesa, $orderType, $IDSTCOMANDA, $qtdPeople);

                $this->orderService->updateStatusTable($cdFilial, $nrMesa, $cdLoja);
                $tableStatus = $this->orderService->checkTableStatus($cdFilial, $nrMesa, $cdLoja);

                if (isset($tableStatus['statusId']) && $tableStatus['statusId'] != 'A')
                    throw new \Exception("Falha na abertura da mesa {$nrMesa}.");

                $modImpressao = 'M';
            } elseif ($type == 'CMD' && !empty($comanda)) {
                $orderData['NRVENDAREST'] = $comanda['NRVENDAREST'];
                $orderData['NRCOMANDA']   = $comanda['NRCOMANDA'];
                $dsComanda                = $comanda['DSCOMANDA'];

                if ($comanda['NRMESA'] != $nrMesa)
                    $this->orderService->updateTableOrder($nrMesa, $orderData['NRVENDAREST']);
                
                if (!empty($qtdPeople) && ($comanda['NRPESMESAVEN'] < $qtdPeople)) 
                    $this->orderService->updatePeopleQuantity($qtdPeople, $orderData['NRVENDAREST']);

            } elseif ($type == 'MES' && !empty($mesa)) {
                $orderData['NRVENDAREST'] = $mesa['NRVENDAREST'];
                $orderData['NRCOMANDA']   = $mesa['NRCOMANDA'];
                $dsComanda                = $mesa['DSCOMANDA'];

                if (!empty($qtdPeople) && ($mesa['NRPESMESAVEN'] < $qtdPeople))
                    $this->orderService->updatePeopleQuantity($qtdPeople, $orderData['NRVENDAREST']);

            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            $this->util->writeLog('order_error', $fileName, $msg);
            return array(
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            );
        }

        // Lançamento de Produtos
        $this->connection->beginTransaction();

        try {
            $pagerId = (string)mt_rand(1000, 9999);
            $sellerPos = $generalParams['CDCAIXA'];
            $items = $this->productService->processProducts($order['items'], $orderType, $cdFilial, $orderData['NRVENDAREST'], $orderData['NRCOMANDA'], $cdLoja, $idControProd, $sellerPos, $generalParams['CDVENDEDOR'], $pagerId);
            $arrayNrseqprodcomven = [];

            if(!empty($items['ITCOMANDAVEN'])) {
                foreach ($items['ITCOMANDAVEN'] as $item) {

                    $nrProdComVen = $this->productService->insertProduct($item, $generalParams);
                    //Observações da OBSITCOMANDAVEN
                    
                    if (!empty($item['ITEM_OBSERVATION'])) {
                        foreach ($item['ITEM_OBSERVATION'] as $itemObservation) {
                            $this->productService->insertObservation($itemObservation, $generalParams, $orderData);
                        }
                    }

                    array_push($arrayNrseqprodcomven, $nrProdComVen);
                }
            }

            $isRodizio = count($items['ITCOMANDAEST']) > 0 ? true : false;
            if($isRodizio) {
                foreach ($items['ITCOMANDAEST'] as $item) {
                    $this->productService->insertComandaEst($item, $generalParams);
                    //Observações da OBSITCOMANDAEST
                    if (!empty($item['ITEM_OBSERVATION'])) {
                        foreach ($item['ITEM_OBSERVATION'] as $itemObservation) {
                            $this->productService->insertObservation($itemObservation, $generalParams, $orderData);
                        }
                    }

                    if(!in_array($item['NRPRODCOMVEN'], $arrayNrseqprodcomven)) 
                        array_push($arrayNrseqprodcomven, $item['NRPRODCOMVEN']);
                }
            }

            if ($mode == "MESA") {
                $modImpressao = 'M';
            } elseif ($mode == "COMANDA") {
                $modImpressao = 'C';
            }

            $dsComanda = isset($dsComanda) ? $dsComanda : $orderData['DSCOMANDA'];
            $arrayItems = $this->orderService->getOrderData($orderData['NRVENDAREST'], $orderData['NRCOMANDA'], $arrayNrseqprodcomven, $isRodizio, $items['ITCOMANDAEST']);
            $printItems = $this->formatItemsToPrint($arrayItems);

            $toGo = 'N';
            $toGoString = ($toGo == 'S') ? '' : '';

            $this->printService->imprimePedido($cdFilial, $cdLoja, $printItems, $generalParams['CDVENDEDOR'], $dsComanda, $nrMesa, $pagerId, $modImpressao, $toGoString, false, $orderData['NRCOMANDA'], null, $sellerPos, 'N', null);

            $this->connection->commit();
            $fileName = date("Ymd-h-m-s") . " - " . $pagerId . ".txt";
            $saveLogOrder = $this->util->writeLog('order_processed', $fileName, $order, true);
            $this->util->finish($order['id'] . ".txt", 'pending');
            $dateLog = date("d/m/Y H:i:s");

            return array(
                'success' => true,
                'message' => "",
                'result'  => []
            );
        } catch (\Exception $e) {
            $this->connection->rollback();
            $msg = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            $this->util->writeLog('order_error', $fileName, $msg);
            return array(
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            );
        }
    }

    public function formatItemsToPrint($items)
    {

        $formatedItems = array();
        foreach ($items as $item) {
            if ($item['IDOBSERVACAOAD'] === 'N') {
                $formatedItem = array();
                $childItems   = array();
                if (!empty($item['ITEMCOMBO'])) {

                    $formatedItem['NMPRODPROMOCAO'] = $item['NMPRODUTO'];
                    $formatedItem['IDTIPOCOMPPROD'] = '3';
                    $formatedItem['ORDEMIMP']       = 0;
                    $formatedItem['IDIMPPROMOCAO']  = 'S';
                    $formatedItem['NRSEQPRODCOM']  = $item['NRSEQPRODCOM'];

                    $childItems = $this->formatItemsToPrint($item['ITEMCOMBO']);
                } else {
                    $formatedItem['IDIMPPROMOCAO']  = 'N';
                    $formatedItem['IDTIPOCOMPPROD'] = null;
                    $formatedItem['CDPRODPROMOCAO'] = $item['CDPRODPROMOCAO'];
                }

                $formatedItem['CDPRODUTO']      = $item['CDPRODUTO'];
                $formatedItem['NRLUGARMESA']    = 1;
                $formatedItem['CDPRODUTO']      = $item['CDPRODUTO'];
                $formatedItem['NMPRODUTO']      = $item['NMPRODUTO'];
                $formatedItem['QTPRODCOMVEN']   = $this->util->toFloat($item['QTPRODCOMVEN']);
                $formatedItem['IDIMPRODUVEZ']   = $item['IDIMPRODUVEZ'];
                $formatedItem['IDPESAPROD']     = $item['IDPESAPROD'];
                $formatedItem['ATRASOPROD']     = null;
                $TXPRODCOMVEN = '';
                $note = '';
                $note = !empty($item['TXPRODCOMVEN']) ? $item['TXPRODCOMVEN'] : '';

                if (!empty($item['TXPRODCOMVEN'])) {
                    $arrayObs = explode("; ", $item['TXPRODCOMVEN']);
                    foreach ($arrayObs as $obs) {
                        if (strpos($obs, ',') == true) {
                            $arrayNotes = explode(",", $obs);
                            foreach ($arrayNotes as $note) {
                                if (!empty($note) && $note != " ") {
                                    $TXPRODCOMVEN .= "{$note};";
                                }
                            }
                        } elseif (!empty($obs) && $obs != " ") {
                            $TXPRODCOMVEN .= "{$obs};";
                        }
                    }
                }
                
                $formatedItem['TXPRODCOMVEN'] = $TXPRODCOMVEN;
                array_push($formatedItems, $formatedItem);

                if (!empty($childItems))
                    $formatedItems = array_merge($formatedItems, $childItems);
            }
        }
        
        return $formatedItems;
    }

    public function validateSend($produtsPartner)
    {
        $generalParams = $this->getGeneralParams();
        $getProducts = $this->productService->ProductsForShipping($generalParams['CDFILIAL'], $produtsPartner);

        return $getProducts;
    }

    public function ProductsToUpdating($produtsPartner)
    {
        $generalParams = $this->getGeneralParams();
        $getProducts = $this->productService->ProductsToUpdating($generalParams['CDFILIAL'], $produtsPartner);

        return $getProducts;
    }

    public function ValidateCards($comandas)
    {
        $generalParams = $this->getGeneralParams();
        $ValidateCards = $this->orderService->ValidateCards($generalParams, $comandas);

        return $ValidateCards;
    }

    public function CanceledProducts()
    {

        global $CLIENT_NAME;
        $configs = $CLIENT_NAME;

        $products = $this->orderService->getCanceledProducts();
        if (!empty($products)) {
            foreach ($products as $product) {
                $fileName = date("Ymd") . "-{$product['NRVENDAREST']}-{$product['NRCOMANDA']}-{$product['NRPRODCOMVEN']}";

                $saveLog = $this->util->writeLog('canceled_received', $fileName, $product);
                if (!$saveLog)
                    $this->util->writeLog('canceled_pending', $fileName, $product);
            }
        }

        $pathPending = $this->logService . "/canceled/pending/";
        $pathReceived = $this->logService . "/canceled/received/" . date("Y/m/d");

        if (!file_exists($pathPending)) {
            mkdir($pathPending, 0775, true);
        }
        $files = scandir($pathPending);

        $files = array_diff($files, array('.', '..'));

        foreach ($files as $file) {
            $arq = file_get_contents($pathPending . $file);
            $productDetails = json_decode($arq, true);

            if (empty($productDetails)){
                unlink($pathPending . $file);
                unlink($pathReceived . $file);
                throw new \Exception("Arquivo de cancelamento corrompido ou invalido");
            }

            $idproext = $productDetails['IDPRODEXT'];

            if (strpos($productDetails['IDORGCMDVENDA'], 'CMD') !== false) {
                $mode = 'card';
                $comandaId = intval($productDetails['DSCOMANDA']);
            } elseif (strpos($productDetails['IDORGCMDVENDA'], 'MES') !== false) {
                $mode = 'table';
                $comandaId = intval($productDetails['NRMESA']);
            } else {
                unlink($pathPending . $file);
                continue;
            }

            $body = [
                "quantity" => intval($productDetails['QTPRODCOMVEN'])
            ];

            $route = "{$mode}/{$comandaId}/item/{$idproext}/cancel";

            $getCards = json_decode($this->util->request('PUT', $route, $body), true);

            if ($getCards['success']) {
                unlink($pathPending . $file);
                $this->util->writeLog('canceled_processed', $file, $productDetails);
                $this->util->writeLog('execution', '', date("d:m:Y H:i:s") . " - " . "Produto {$productDetails['NMPRODUTO']} qtd: {$productDetails['QTPRODCOMVEN']} - cancelado na comanda {$productDetails['DSCOMANDA']}. ");
            }else {
                unlink($pathPending . $file);
            }
        }

        return null;
    }

    public function orderSynchronization()
    {
        $productDetails = $this->orderService->getUnsyncedProducts();
        if (!empty($productDetails)) {
            foreach ($productDetails as $product) {
                $vrprecitem = $this->util->toFloat($product['VRPRECCOMVEN']);
                $vrDesconto = !empty($this->util->toFloat($product['VRDESCCOMVEN'])) ? $this->util->toFloat($product['VRDESCCOMVEN']) : 0;
                $vrAcr      = !empty($this->util->toFloat($product['VRACRCOMVEN'])) ? $this->util->toFloat($product['VRACRCOMVEN']) : 0;
                $price      = (($vrprecitem + $vrAcr) - $vrDesconto);

                $body = [
                    'code'     => $product['CDPRODUTO'],
                    'name'     => $product['NMPRODUTO'],
                    'quantity' => intval($product['QTPRODCOMVEN']),
                    'price'    => $this->util->toFloat($price)
                ];

                if (strpos($product['IDORGCMDVENDA'], 'CMD') !== false) {
                    $mode = 'card';
                    $comandaId = intval($product['DSCOMANDA']);
                } elseif (strpos($product['IDORGCMDVENDA'], 'MES') !== false) {;
                    $mode = 'table';
                    $comandaId = intval($product['NRMESA']);
                } else {
                    continue;
                }

                $route = "{$mode}/{$comandaId}/item";

                $sendProducts = json_decode($this->util->request('POST', $route, $body), true);

                if ($sendProducts['success']) {
                    $this->util->writeLog('execution', '', date("d:m:Y H:i:s") . " - " . "Produto {$product['NMPRODUTO']} - inserido na comanda {$product['DSCOMANDA']} no CD Abrahao. ");

                    $idproext = $sendProducts['data']['id'];
                    $this->orderService->updateComandaVen($product, $idproext);
                }
            }
        }

        return null;
    }

    public function closeOrder()
    {
        $sales = $this->orderService->getsales();
        
        if (!empty($sales)) {
            foreach ($sales as $sale) {
                $fileName = date("Ymd") . "-{$sale['NRVENDAREST']}-{$sale['DSCOMANDAVND']}";

                $saveLog = $this->util->writeLog('close_received', $fileName, $sale);
                if (!$saveLog)
                    $this->util->writeLog('close_pending', $fileName, $sale);
            }
        }


        $pathPending = $this->logService . "/close/pending/";
        $pathReceived = $this->logService . "/close/received/" . date("Y/m/d");
        $pathProcessed = $this->logService . "/close/processed/" . date("Y/m/d");

        if (!file_exists($pathPending)) {
            mkdir($pathPending, 0775, true);
        }

        $files = scandir($pathPending);

        // remove os itens '.' e '..' do array
        $files = array_diff($files, array('.', '..'));

        // exibe o nome de cada arquivo
        foreach ($files as $file) {
            $arq = file_get_contents($pathPending . $file);
            $saleData = json_decode($arq, true);

            if(empty($saleData)){
                unlink($pathPending . $file);
                throw new \Exception("Arquivo de finalização de pedido corrompido ou invalido");
            }

            $serviceTax = $this->util->toFloat($saleData['VRTXSEVENDA']);

            if (strpos($saleData['IDORIGEMVENDA'], 'CMD') !== false) {
                $mode = 'card';
                $comandaId = intval($saleData['DSCOMANDAVND']);
            } elseif (strpos($saleData['IDORIGEMVENDA'], 'MES') !== false) {
                $mode = 'table';
                $comandaId = intval($saleData['NRMESA']);
            } else {
                unlink($pathPending . $file);
                continue;
            }

            $route = "{$mode}/{$comandaId}/close";

            $closeOrder = json_decode($this->util->request('PUT', $route, null), true);
            
            if (!$closeOrder['success']) {
                unlink($pathPending . $file);
                throw new \Exception("Erro na finalização da comanda/mesa: {$comandaId} - {$closeOrder['message']}");
            }

            unlink($pathPending . $file);
            $this->util->writeLog('close_processed', $file, $saleData);
            $this->util->writeLog('execution', '', date("d:m:Y H:i:s") . " - " . "Comanda {$saleData['DSCOMANDAVND']} - Venda{$saleData['NRSEQVENDA']} finalizada no CD abrahao.");
        }

        return null;
    }

    public function transferProducts()
    {
        global $CLIENT_NAME;
        $configs = $CLIENT_NAME;

        $products = $this->orderService->getTransferProduct();
        if (!empty($products)) {
            foreach ($products as $product) {
                $fileName = date("Ymd") . "-{$product['NRVENDAREST']}-{$product['NRCOMANDA']}-{$product['NRPRODCOMVEN']}";

                $saveLog = $this->util->writeLog('transfer_received', $fileName, $product);
                if (!$saveLog)
                    $this->util->writeLog('transfer_pending', $fileName, $product);
            }
        }

        $pathPending = $this->logService . "/transfer/pending/";

        if (!file_exists($pathPending)) {
            mkdir($pathPending, 0775, true);
        }
        $files = scandir($pathPending);

        $files = array_diff($files, array('.', '..'));

        foreach ($files as $file) {
            $arq = file_get_contents($pathPending . $file);
            $productDetails = json_decode($arq, true);

            if(empty($saleData))
                throw new \Exception("Arquivo de transferencia de produtos corrompido ou invalido");

            $idproext = $productDetails['IDPRODEXT'];
            $productOrigin = intval($productDetails['NRMESADSCOMORIT']);

            if (strpos($productDetails['IDORGCMDVENDA'], 'CMD') !== false) {
                $mode = 'card';
                $comandaId = intval($productDetails['DSCOMANDA']);
            } elseif (strpos($productDetails['IDORGCMDVENDA'], 'MES') !== false) {
                $mode = 'table';
                $comandaId = intval($productDetails['NRMESA']);
            } else {
                unlink($pathPending . $file);
                continue;
            }

            $body = [
                'new_table' => intval($comandaId),
                "quantity"  => intval($productDetails['QTPRODCOMVEN'])
            ];

            $route = "{$mode}/{$comandaId}/item/{$idproext}/transfer";

            $getCards = json_decode($this->util->request('PUT', $route, $body), true);

            if ($getCards['success']) {
                unlink($pathPending . $file);
                $this->util->writeLog('transfer_processed', $file, $productDetails);
                $this->util->writeLog('execution', '', date("d:m:Y H:i:s") . " - " . "Produto {$productDetails['NMPRODUTO']} qtd: {$productDetails['QTPRODCOMVEN']} - transferido da mesa/comanda {$productOrigin} para {$comandaId}. ");
            }
        }

        return null;
    }
}
